#!/usr/bin/perl
# j5_xml_rpc_client_get_last_updated_user_files.pl
# Nathan Hillson 01-SEP-15

# *** Copyright Notice ***
#
# Copyright (c) 2010-2015, The Regents of the University of California,
# through Lawrence Berkeley National Laboratory (subject to receipt of
# any required approvals from the U.S. Dept. of Energy). All rights
# reserved.
#
# NOTICE. This software was developed under funding from the U.S. Department of Energy.
# As such, the U.S. Government has been granted for itself and others acting on its
# behalf a paid-up, nonexclusive, irrevocable, worldwide license in the Software to
# reproduce, prepare derivative works, and perform publicly and display publicly.
# Beginning five (5) years after the date permission to assert copyright is obtained
# from the U.S. Department of Energy, and subject to any subsequent five (5) year
# renewals, the U.S. Government is granted for itself and others acting on its behalf a
# paid-up, nonexclusive, irrevocable, worldwide license in the Software to reproduce,
# prepare derivative works, distribute copies to the public, perform publicly and
# display publicly, and to permit others to do so. 
#
# This program is free software; you can redistribute it and/or modify
# it under the same terms as the Perl "Artistic License"
# (http://dev.perl.org/licenses/artistic.html).
#
# *** End of Copyright Notice ***

use XML::RPC;
use MIME::Base64;
require LWP::UserAgent;

if (@ARGV != 1) {
    die "Usage: $0 j5_session_id\n";
}

my $j5_session_id = $ARGV[0];
my $xmlrpc;
my $result;
my $ua;

# setting the user agent timeout to a longer period of time is required to make sure that j5 has sufficient time to complete an assembly
$ua = LWP::UserAgent->new();
$ua->timeout( 86400 ); # wait up until 24 hours for j5 to complete assembly, then bail

$xmlrpc = XML::RPC->new('https://j5.jbei.org/bin/j5_xml_rpc.pl',lwp_useragent => $ua);

$result = $xmlrpc->call( 'GetLastUpdatedUserFiles', { j5_session_id => $ARGV[0] } );

if (defined $result->{"faultString"}) {
  print "Error (GetLastUpdatedUserFiles): faultCode: ",$result->{"faultCode"}," faultString: ",$result->{"faultString"},"\n";
}

else {
  my $output_file_handle;
  
  print $result->{"username"}," ",$result->{"j5_parameters_file_exists"}," ",$result->{"downstream_automation_parameters_file_exists"}," ",
    $result->{"sequences_list_file_exists"}," ",
    $result->{"zipped_sequences_file_exists"}," ",$result->{"parts_list_file_exists"}," ",$result->{"target_part_order_list_file_exists"}," ",
    $result->{"eugene_rules_list_file_exists"}," ",$result->{"master_plasmids_file_exists"}," ",$result->{"master_oligos_file_exists"}," ",
    $result->{"master_direct_syntheses_file_exists"},"\n";

  if ($result->{"j5_parameters_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">j5parameters.csv") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_j5_parameters_file"});
    close($output_file_handle);
  }

  if ($result->{"downstream_parameters_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">downstreamautomationparameters.csv") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_downstream_automation_parameters_file"});
    close($output_file_handle);
  }
  
  if ($result->{"sequences_list_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">mastersequences.csv") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_sequences_list_file"});
    close($output_file_handle);
  }
  
  if ($result->{"zipped_sequences_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">masterzippedsequences.zip") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_zipped_sequences_file"});
    close($output_file_handle);
  }

  if ($result->{"parts_list_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">partslist.csv") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_parts_list_file"});
    close($output_file_handle);
  }
  
  if ($result->{"target_part_order_list_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">targetpartsorder.csv") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_target_part_order_list_file"});
    close($output_file_handle);
  }
  
  if ($result->{"eugene_rules_list_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">eugeneruleslist.eug") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_eugene_rules_list_file"});
    close($output_file_handle);
  }

  if ($result->{"master_plasmids_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">masterplasmidlist.csv") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_master_plasmids_file"});
    close($output_file_handle);
  }
  
  if ($result->{"master_oligos_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">masteroligolist.csv") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_master_oligos_file"});
    close($output_file_handle);
  }
  
  if ($result->{"master_direct_syntheses_file_exists"}=~/^TRUE/i) {
    open($output_file_handle, ">masterdirectsyntheseslist.csv") or die "$!";
    print $output_file_handle decode_base64($result->{"encoded_master_direct_syntheses_file"});
    close($output_file_handle);
  }
}
